<?php
declare(strict_types=1);

namespace Catequil\Webshop\Model;

use Catequil\Db\Connection;
use Catequil\Overseas\Model\ApiKey;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Embedded;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;
use Laminas\ServiceManager\AbstractFactory\ConfigAbstractFactory;

/**
 * Class DefaultWebshop
 *
 * @package Catequil\Webshop\Model
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
#[Entity]
#[Table('webshop', options: ['collation' => 'utf8mb4_croatian_ci'])]
class Webshop implements WebshopInterface
{
    //<editor-fold desc="Entity Fields">

    #[Id]
    #[GeneratedValue('AUTO')]
    #[Column(type: Types::BIGINT, options: ['unsigned' => true])]
    protected ?int $id = null;
    #[Column(type: Types::STRING, length: 191, unique: true)]
    protected string $domain = '';
    #[Embedded(ConnectionParams::class, 'db_')]
    protected ConnectionParams $connectionParams;
    #[Column(type: Types::STRING, nullable: true)]
    protected ?string $sshHost = null;
    #[Column(type: Types::STRING, nullable: true)]
    protected ?string $sshUser = null;
    #[Column(type: Types::STRING, nullable: true)]
    protected ?string $sshPassword = null;
    #[Column(type: Types::STRING, nullable: true)]
    protected ?string $clearCacheCommand = null;
    #[Column(type: Types::JSON)]
    protected array $categoryMap = [];
    #[ManyToOne(targetEntity: ApiKey::class)]
    #[JoinColumn('overseas_api_key_id')]
    protected ?ApiKey $overseasApiKey = null;
    #[Column(type: Types::BOOLEAN)]
    protected bool $packOrders = false;

    //</editor-fold>

    private ?Connection $conn = null;

    public function __construct()
    {
        $this->connectionParams = new ConnectionParams();
    }

    //<editor-fold desc="Getters & Setters">

    /**
     * @return int|null
     */
    public function getId(): ?int
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getDomain(): string
    {
        return $this->domain;
    }

    /**
     * @param string $domain
     * @return void
     */
    public function setDomain(string $domain): void
    {
        $this->domain = $domain;
    }

    /**
     * @return string|null
     */
    public function getSshHost(): ?string
    {
        return $this->sshHost;
    }

    /**
     * @return string|null
     */
    public function getSshUser(): ?string
    {
        return $this->sshUser;
    }

    /**
     * @return string|null
     */
    public function getSshPassword(): ?string
    {
        return $this->sshPassword;
    }

    /**
     * @return string|null
     */
    public function getClearCacheCommand(): ?string
    {
        return $this->clearCacheCommand;
    }

    /**
     * @return array
     */
    public function getCategoryMap(): array
    {
        return $this->categoryMap;
    }

    /**
     * @return ApiKey|null
     */
    public function getOverseasApiKey(): ?ApiKey
    {
        return $this->overseasApiKey;
    }

    /**
     * @return bool
     */
    public function isPackOrders(): bool
    {
        return $this->packOrders;
    }

    /**
     * @return Connection
     * @noinspection PhpDocMissingThrowsInspection
     */
    public function getConnection(): Connection
    {
        if ($this->conn === null) {
            $factory = new ConfigAbstractFactory();
            /** @noinspection PhpFieldAssignmentTypeMismatchInspection */
            /** @noinspection PhpUnhandledExceptionInspection */
            $this->conn = DriverManager::getConnection(
                $this->connectionParams->getArrayCopy()
            );
        }

        return $this->conn;
    }

    /**
     * @param string $name
     * @param null $default
     * @return false|mixed
     * @throws DBALException
     */
    public function getOption(string $name, $default = null): mixed
    {
        $conn = $this->getConnection();
        /** @noinspection SqlResolve */
        /** @noinspection SqlNoDataSourceInspection */
        $option = $conn->fetchOne('SELECT option_value FROM wp_options WHERE option_name = ?', [$name]);

        $unserialized = is_string($option) ? @unserialize($option) : false;
        return $unserialized !== false ? $unserialized : ($option ?: $default);
    }

    public function setCategoryMap(array $categoryMap): void
    {
        $this->categoryMap = $categoryMap;
    }

    public function getMappedCategoryCount(): int
    {
        return count($this->categoryMap);
    }

    public function getMappedCategoryId(int $categoryId): ?int
    {
        return $this->categoryMap[$categoryId] ?? null;
    }

    //</editor-fold>
}